==> zio-aws-wafv2_2.13-5.17.128.1-sources.jar: zio/aws/wafv2/model/RateBasedStatement.scala <==
package zio.aws.wafv2.model
import zio.ZIO
import zio.aws.core.{AwsError, BuilderHelper}
import zio.aws.wafv2.model.primitives.RateLimit
import scala.jdk.CollectionConverters._
final case class RateBasedStatement(
    limit: RateLimit,
    aggregateKeyType: zio.aws.wafv2.model.RateBasedStatementAggregateKeyType,
    scopeDownStatement: Option[zio.aws.wafv2.model.Statement] = None,
    forwardedIPConfig: Option[zio.aws.wafv2.model.ForwardedIPConfig] = None
) {
  def buildAwsValue()
      : software.amazon.awssdk.services.wafv2.model.RateBasedStatement = {
    import RateBasedStatement.zioAwsBuilderHelper.BuilderOps
    software.amazon.awssdk.services.wafv2.model.RateBasedStatement
      .builder()
      .limit(RateLimit.unwrap(limit): java.lang.Long)
      .aggregateKeyType(aggregateKeyType.unwrap)
      .optionallyWith(scopeDownStatement.map(value => value.buildAwsValue()))(
        _.scopeDownStatement
      )
      .optionallyWith(forwardedIPConfig.map(value => value.buildAwsValue()))(
        _.forwardedIPConfig
      )
      .build()
  }
  def asReadOnly: zio.aws.wafv2.model.RateBasedStatement.ReadOnly =
    zio.aws.wafv2.model.RateBasedStatement.wrap(buildAwsValue())
}
object RateBasedStatement {
  private lazy val zioAwsBuilderHelper: BuilderHelper[
    software.amazon.awssdk.services.wafv2.model.RateBasedStatement
  ] = BuilderHelper.apply
  trait ReadOnly {
    def asEditable: zio.aws.wafv2.model.RateBasedStatement =
      zio.aws.wafv2.model.RateBasedStatement(
        limit,
        aggregateKeyType,
        scopeDownStatement.map(value => value.asEditable),
        forwardedIPConfig.map(value => value.asEditable)
      )
    def limit: RateLimit
    def aggregateKeyType: zio.aws.wafv2.model.RateBasedStatementAggregateKeyType
    def scopeDownStatement: Option[zio.aws.wafv2.model.Statement.ReadOnly]
    def forwardedIPConfig
        : Option[zio.aws.wafv2.model.ForwardedIPConfig.ReadOnly]
    def getLimit: ZIO[Any, Nothing, RateLimit] = ZIO.succeed(limit)
    def getAggregateKeyType: ZIO[
      Any,
      Nothing,
      zio.aws.wafv2.model.RateBasedStatementAggregateKeyType
    ] = ZIO.succeed(aggregateKeyType)
    def getScopeDownStatement
        : ZIO[Any, AwsError, zio.aws.wafv2.model.Statement.ReadOnly] =
      AwsError.unwrapOptionField("scopeDownStatement", scopeDownStatement)
    def getForwardedIPConfig
        : ZIO[Any, AwsError, zio.aws.wafv2.model.ForwardedIPConfig.ReadOnly] =
      AwsError.unwrapOptionField("forwardedIPConfig", forwardedIPConfig)
  }
  private final class Wrapper(
      impl: software.amazon.awssdk.services.wafv2.model.RateBasedStatement
  ) extends zio.aws.wafv2.model.RateBasedStatement.ReadOnly {
    override val limit: RateLimit =
      zio.aws.wafv2.model.primitives.RateLimit(impl.limit())
    override val aggregateKeyType
        : zio.aws.wafv2.model.RateBasedStatementAggregateKeyType =
      zio.aws.wafv2.model.RateBasedStatementAggregateKeyType
        .wrap(impl.aggregateKeyType())
    override val scopeDownStatement
        : Option[zio.aws.wafv2.model.Statement.ReadOnly] = scala
      .Option(impl.scopeDownStatement())
      .map(value => zio.aws.wafv2.model.Statement.wrap(value))
    override val forwardedIPConfig
        : Option[zio.aws.wafv2.model.ForwardedIPConfig.ReadOnly] = scala
      .Option(impl.forwardedIPConfig())
      .map(value => zio.aws.wafv2.model.ForwardedIPConfig.wrap(value))
  }
  def wrap(
      impl: software.amazon.awssdk.services.wafv2.model.RateBasedStatement
  ): zio.aws.wafv2.model.RateBasedStatement.ReadOnly = new Wrapper(impl)
}

==> zio-aws-wafv2_2.13-5.19.17.2-sources.jar: zio/aws/wafv2/model/RateBasedStatement.scala <==
package zio.aws.wafv2.model
import zio.ZIO
import zio.aws.core.{AwsError, BuilderHelper}
import zio.prelude.data.Optional
import zio.aws.wafv2.model.primitives.RateLimit
import scala.jdk.CollectionConverters._
final case class RateBasedStatement(
    limit: RateLimit,
    aggregateKeyType: zio.aws.wafv2.model.RateBasedStatementAggregateKeyType,
    scopeDownStatement: Optional[zio.aws.wafv2.model.Statement] =
      Optional.Absent,
    forwardedIPConfig: Optional[zio.aws.wafv2.model.ForwardedIPConfig] =
      Optional.Absent
) {
  def buildAwsValue()
      : software.amazon.awssdk.services.wafv2.model.RateBasedStatement = {
    import RateBasedStatement.zioAwsBuilderHelper.BuilderOps
    software.amazon.awssdk.services.wafv2.model.RateBasedStatement
      .builder()
      .limit(RateLimit.unwrap(limit): java.lang.Long)
      .aggregateKeyType(aggregateKeyType.unwrap)
      .optionallyWith(scopeDownStatement.map(value => value.buildAwsValue()))(
        _.scopeDownStatement
      )
      .optionallyWith(forwardedIPConfig.map(value => value.buildAwsValue()))(
        _.forwardedIPConfig
      )
      .build()
  }
  def asReadOnly: zio.aws.wafv2.model.RateBasedStatement.ReadOnly =
    zio.aws.wafv2.model.RateBasedStatement.wrap(buildAwsValue())
}
object RateBasedStatement {
  private lazy val zioAwsBuilderHelper: BuilderHelper[
    software.amazon.awssdk.services.wafv2.model.RateBasedStatement
  ] = BuilderHelper.apply
  trait ReadOnly {
    def asEditable: zio.aws.wafv2.model.RateBasedStatement =
      zio.aws.wafv2.model.RateBasedStatement(
        limit,
        aggregateKeyType,
        scopeDownStatement.map(value => value.asEditable),
        forwardedIPConfig.map(value => value.asEditable)
      )
    def limit: RateLimit
    def aggregateKeyType: zio.aws.wafv2.model.RateBasedStatementAggregateKeyType
    def scopeDownStatement: Optional[zio.aws.wafv2.model.Statement.ReadOnly]
    def forwardedIPConfig
        : Optional[zio.aws.wafv2.model.ForwardedIPConfig.ReadOnly]
    def getLimit: ZIO[Any, Nothing, RateLimit] = ZIO.succeed(limit)
    def getAggregateKeyType: ZIO[
      Any,
      Nothing,
      zio.aws.wafv2.model.RateBasedStatementAggregateKeyType
    ] = ZIO.succeed(aggregateKeyType)
    def getScopeDownStatement
        : ZIO[Any, AwsError, zio.aws.wafv2.model.Statement.ReadOnly] =
      AwsError.unwrapOptionField("scopeDownStatement", scopeDownStatement)
    def getForwardedIPConfig
        : ZIO[Any, AwsError, zio.aws.wafv2.model.ForwardedIPConfig.ReadOnly] =
      AwsError.unwrapOptionField("forwardedIPConfig", forwardedIPConfig)
  }
  private final class Wrapper(
      impl: software.amazon.awssdk.services.wafv2.model.RateBasedStatement
  ) extends zio.aws.wafv2.model.RateBasedStatement.ReadOnly {
    override val limit: RateLimit =
      zio.aws.wafv2.model.primitives.RateLimit(impl.limit())
    override val aggregateKeyType
        : zio.aws.wafv2.model.RateBasedStatementAggregateKeyType =
      zio.aws.wafv2.model.RateBasedStatementAggregateKeyType
        .wrap(impl.aggregateKeyType())
    override val scopeDownStatement
        : Optional[zio.aws.wafv2.model.Statement.ReadOnly] =
      zio.aws.core.internal
        .optionalFromNullable(impl.scopeDownStatement())
        .map(value => zio.aws.wafv2.model.Statement.wrap(value))
    override val forwardedIPConfig
        : Optional[zio.aws.wafv2.model.ForwardedIPConfig.ReadOnly] =
      zio.aws.core.internal
        .optionalFromNullable(impl.forwardedIPConfig())
        .map(value => zio.aws.wafv2.model.ForwardedIPConfig.wrap(value))
  }
  def wrap(
      impl: software.amazon.awssdk.services.wafv2.model.RateBasedStatement
  ): zio.aws.wafv2.model.RateBasedStatement.ReadOnly = new Wrapper(impl)
}
